# Create StoragePools and VirtualDisks
# created by c.rachfahl@rachfahl.de 
# Version 1.0
# You are free to use it but on your own risk

$FSCluster = @{
            ClusterName = "FSCluster";
            ClusterNodes = ("FSNode1","FSNode2");
            ClusterAddress = "192.168.210.97";
}


$pool1 = @{poolName = "Pool1";
            diskSlotList = (1..24 );
            dataCopies = 2;
            columnCount = 2;
            vdCount = 2;
            EnclosureCount = 1;
            vdFriendlyNames = ("VirtualDisk1", "VirtualDisk2");
            Quorum = (5GB, "FSQuorum", 0, 2)
}

$pool2 = @{poolName = "Pool2";
            diskSlotList = (5..10);
            dataCopies = 2;
            columnCount = 5;
            vdCount = 2;
            EnclosureCount = 3;
            vdFriendlyNames = ("VirtualDisk3", "VirtualDisk4");
            Quorum = (15GB, "HVQuorum", 1, 3)
}

Function ComputeSSDTierSize
{
    param($disks,
    $dataCopies,
    $vdCount,
    $EnclosureCount,
    $ReserveinGB)

    # Get raw space
    $ssdSpace = 0
    $diskCount = 0

    foreach ($disk in $disks)
    {
        if ($disk.MediaType -eq "SSD")
        {
            $ssdSpace += $disk.Size
            $diskCount++
        }

    }
    
    if($diskCount -ne 0) {
        $ReserveSpace = ((($ssdSpace/$diskCount) + 8GB) * $EnclosureCount) + ($ReserveinGB  * $dataCopies)
        return ($ssdSpace - $ReserveSpace) / $dataCopies / $vdCount
    } else {
        return 0
    }
}

Function ComputeHDDTierSize
{
    param($disks,
    $dataCopies,
    $vdCount,
    $EnclosureCount,
    $ReserveinGB)

    # Get raw space
    $hddSpace = 0
    $diskCount = 0

    foreach ($disk in $disks)
    {
        if ($disk.MediaType -eq "HDD")
        {
            $hddSpace += $disk.Size
            $diskCount++
        }
    }

    if($diskCount -ne 0) {
        $ReserveSpace = ((($hddSpace/$diskCount) + 8GB) * $EnclosureCount) + ($ReserveinGB * $dataCopies)
        return ($hddSpace - $ReserveSpace) / $dataCopies / $vdCount
    } else {
        return 0
    }
}


Function CreatePools
{
    param($poolName,
    $diskSlotList,
    $dataCopies,
    $columnCount,
    $vdCount,
    $EnclosureCount,
    $vdFriendlyNames,
    $Quorum
    )

    # Variables
    $QuorumSize = $Quorum[0]
    $QuorumName = $Quorum[1]
    $QuorumIsCSV = $Quorum[2]
    $QuorumDataCopies = $Quorum[3]
    $disksToPool = @()

    $ComputerName = $env:COMPUTERNAME

    # Move Cluster Group "Cluster Group" to Computername
    Move-ClusterGroup -Name "Cluster Group" -Node $ComputerName

    foreach ($diskSlotNumber in $diskSlotList)
    {
        $disk = Get-PhysicalDisk | where SlotNumber -EQ $diskSlotNumber
        if ($disk.EnclosureNumber -ne $null) {
            $disksToPool += $disk
        }
    }

    #Create Pool
    New-StoragePool -FriendlyName $poolName -StorageSubSystemFriendlyName "Clustered Storage Spaces*" -PhysicalDisks $disksToPool -WriteCacheSizeDefault 0

    #Rename Pool
    (Get-ClusterResource | where ResourceType -EQ "Storage Pool" | where Name -like "Cluster Pool*").name = $poolName

    #Create Tiers
    $ssdTierName = "SSD-Tier-"+ $poolName
    $hddTierName = "HDD-Tier-" + $poolName

    New-StorageTier -StoragePoolFriendlyName $poolName -FriendlyName $ssdTierName -MediaType SSD
    New-StorageTier -StoragePoolFriendlyName $poolName -FriendlyName $hddTierName -MediaType HDD

    $ssd_tier = Get-StorageTier -FriendlyName $ssdTierName
    $hdd_tier = Get-StorageTier -FriendlyName $hddTierName

    #Calculate VirtualDisks Sizes
    $ssdTierSize = ComputeSSDTierSize -disks $disksToPool -dataCopies $dataCopies -vdCount $vdCount -EnclosureCount $EnclosureCount -ReserveinGB (($vdCount+1)*1024*1024*1024)
    if (($QuorumSize -gt 0) -and ($QuorumDataCopies -gt 0)) {
        $hddTierSize = ComputeHDDTierSize -disks $disksToPool -dataCopies $dataCopies -vdCount $vdCount -EnclosureCount $EnclosureCount -ReserveinGB $QuorumSize
    } else {
        $hddTierSize = ComputeHDDTierSize -disks $disksToPool -dataCopies $dataCopies -vdCount $vdCount -EnclosureCount $EnclosureCount -ReserveinGB 0
    }
    
    for ($k = 0; $k -lt $vdCount; $k++)
    {
        $vdName = $vdFriendlyNames[$k]       

        # Create Virtual Disk
        if($ssdTierSize -gt 0) {
            New-VirtualDisk -StoragePoolFriendlyName $poolName -StorageTiers @($ssd_tier,$hdd_tier) `
                    -StorageTierSizes @($ssdTierSize,$hddTierSize) -ResiliencySettingName Mirror -NumberOfDataCopies $dataCopies `
                    -NumberOfColumns $columnCount -ProvisioningType Fixed -FriendlyName $vdName -Interleave 64KB -IsEnclosureAware:$false -WriteCacheSize 1GB
        } else {
            New-VirtualDisk -StoragePoolFriendlyName $poolName -Size $hddTierSize -ResiliencySettingName Mirror -NumberOfDataCopies $dataCopies `
                    -NumberOfColumns $columnCount -ProvisioningType Fixed -FriendlyName $vdName -Interleave 64KB -IsEnclosureAware:$false -WriteCacheSize 0
        }

        #Move "Availible Storage" ClusterGroup to localhost
        Move-ClusterGroup Available Storage Node $ComputerName
        
        # Rename Virtual Disk
        $clusterResource = Get-ClusterResource -Name "*$vdName*"
        $ClusterResource.Name = $vdName

        # set VirtualDisk to Maintanace Mode 
        Suspend-ClusterResource $clusterResource

        # Create Volume
        Get-VirtualDisk $vdName | Get-Disk | New-Partition UseMaximumSize | Format-Volume FileSystem NTFS AllocationUnitSize 64KB NewFileSystemLabel $vdName Confirm:$false

        # unset VirtualDisk Maintance Mode
        Resume-ClusterResource $clusterResource

        # Promote to ClusterGet-VirtualDiskShareVolume
        Add-ClusterSharedVolume -InputObject $clusterResource
    }

    #Quorum Disk
    if($QuorumSize -gt 0) {
  
        # Create Virtual Disk
        New-VirtualDisk -StoragePoolFriendlyName $poolName -Size $QuorumSize -ResiliencySettingName Mirror -NumberOfDataCopies $QuorumDataCopies `
                    -NumberOfColumns $columnCount -ProvisioningType Fixed -FriendlyName $QuorumName -Interleave 64KB -IsEnclosureAware:$false
        
        #Move "Availible Storage" ClusterGroup to localhost
        Move-ClusterGroup Available Storage Node $ComputerName
        
        # Rename Virtual Disk
        $clusterResource = Get-ClusterResource -Name "*$QuorumName*"
        $ClusterResource.Name = $QuorumName

        # set VirtualDisk to Maintanace Mode 
        Suspend-ClusterResource $clusterResource

        # Create Volume
        Get-VirtualDisk $QuorumName | Get-Disk | New-Partition UseMaximumSize | Format-Volume FileSystem NTFS AllocationUnitSize 64KB NewFileSystemLabel $QuorumName Confirm:$false

        # unset VirtualDisk Maintance Mode
        Resume-ClusterResource $clusterResource

        # should the Quorum be the FS-Cluster Quorum?
        if ($QuorumIsCSV -eq 0) {
            # Set Quorum
            Set-ClusterQuorum -DiskWitness $QuorumName
        } else {
	    # Promote to ClusterSharedVolume
            Add-ClusterSharedVolume -InputObject $clusterResource
        }
    }
}

function CreateCluster {
    param($ClusterName,
    $ClusterNodes,
    $ClusterAddress)

#    Test-Cluster -Node $ClusterNodes
    New-Cluster -Name $ClusterName -Node $ClusterNodes -StaticAddress $ClusterAddress -NoStorage
}

# Create the Failover-Cluster for the Fileserver
CreateCluster @FSCluster

$pools = ($pool1)

foreach ($pool in $pools)
{
    CreatePools @pool
}